# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist

class GetJoinedMembershipUsersResponse(BaseModel):
    """
    List of users who have joined the membership
    https://developers.line.biz/en/reference/messaging-api/#get-membership-user-ids
    """
    user_ids: conlist(StrictStr, max_items=1000) = Field(..., alias="userIds", description="A list of user IDs who joined the membership. Users who have not agreed to the bot user agreement, are not following the bot, or are not active will be excluded. If there are no users in the membership, an empty list will be returned. ")
    next: Optional[StrictStr] = Field(None, description="A continuation token to get next remaining membership user IDs. Returned only when there are remaining user IDs that weren't returned in the userIds property in the previous request. The continuation token expires in 24 hours (86,400 seconds).  ")

    __properties = ["userIds", "next"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetJoinedMembershipUsersResponse:
        """Create an instance of GetJoinedMembershipUsersResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetJoinedMembershipUsersResponse:
        """Create an instance of GetJoinedMembershipUsersResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetJoinedMembershipUsersResponse.parse_obj(obj)

        _obj = GetJoinedMembershipUsersResponse.parse_obj({
            "user_ids": obj.get("userIds"),
            "next": obj.get("next")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt

class GetStatisticsPerUnitResponseMessage(BaseModel):
    """
    GetStatisticsPerUnitResponseMessage
    https://developers.line.biz/en/reference/messaging-api/#get-statistics-per-unit-response
    """
    seq: StrictInt = Field(..., description="Bubble's serial number.")
    impression: Optional[StrictInt] = Field(None, description="Number of times the bubble was displayed.")
    media_played: Optional[StrictInt] = Field(None, alias="mediaPlayed", description="Number of times audio or video in the bubble started playing.")
    media_played25_percent: Optional[StrictInt] = Field(None, alias="mediaPlayed25Percent", description="Number of times audio or video in the bubble started playing and was played 25% of the total time.")
    media_played50_percent: Optional[StrictInt] = Field(None, alias="mediaPlayed50Percent", description="Number of times audio or video in the bubble started playing and was played 50% of the total time.")
    media_played75_percent: Optional[StrictInt] = Field(None, alias="mediaPlayed75Percent", description="Number of times audio or video in the bubble started playing and was played 75% of the total time.")
    media_played100_percent: Optional[StrictInt] = Field(None, alias="mediaPlayed100Percent", description="Number of times audio or video in the bubble started playing and was played 100% of the total time.")
    unique_impression: Optional[StrictInt] = Field(None, alias="uniqueImpression", description="Number of users the bubble was displayed.")
    unique_media_played: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed", description="Number of users that started playing audio or video in the bubble.")
    unique_media_played25_percent: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed25Percent", description="Number of users that started playing audio or video in the bubble and played 25% of the total time.")
    unique_media_played50_percent: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed50Percent", description="Number of users that started playing audio or video in the bubble and played 50% of the total time.")
    unique_media_played75_percent: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed75Percent", description="Number of users that started playing audio or video in the bubble and played 75% of the total time.")
    unique_media_played100_percent: Optional[StrictInt] = Field(None, alias="uniqueMediaPlayed100Percent", description="Number of users that started playing audio or video in the bubble and played 100% of the total time.")

    __properties = ["seq", "impression", "mediaPlayed", "mediaPlayed25Percent", "mediaPlayed50Percent", "mediaPlayed75Percent", "mediaPlayed100Percent", "uniqueImpression", "uniqueMediaPlayed", "uniqueMediaPlayed25Percent", "uniqueMediaPlayed50Percent", "uniqueMediaPlayed75Percent", "uniqueMediaPlayed100Percent"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetStatisticsPerUnitResponseMessage:
        """Create an instance of GetStatisticsPerUnitResponseMessage from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if impression (nullable) is None
        # and __fields_set__ contains the field
        if self.impression is None and "impression" in self.__fields_set__:
            _dict['impression'] = None

        # set to None if media_played (nullable) is None
        # and __fields_set__ contains the field
        if self.media_played is None and "media_played" in self.__fields_set__:
            _dict['mediaPlayed'] = None

        # set to None if media_played25_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.media_played25_percent is None and "media_played25_percent" in self.__fields_set__:
            _dict['mediaPlayed25Percent'] = None

        # set to None if media_played50_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.media_played50_percent is None and "media_played50_percent" in self.__fields_set__:
            _dict['mediaPlayed50Percent'] = None

        # set to None if media_played75_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.media_played75_percent is None and "media_played75_percent" in self.__fields_set__:
            _dict['mediaPlayed75Percent'] = None

        # set to None if media_played100_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.media_played100_percent is None and "media_played100_percent" in self.__fields_set__:
            _dict['mediaPlayed100Percent'] = None

        # set to None if unique_impression (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_impression is None and "unique_impression" in self.__fields_set__:
            _dict['uniqueImpression'] = None

        # set to None if unique_media_played (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played is None and "unique_media_played" in self.__fields_set__:
            _dict['uniqueMediaPlayed'] = None

        # set to None if unique_media_played25_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played25_percent is None and "unique_media_played25_percent" in self.__fields_set__:
            _dict['uniqueMediaPlayed25Percent'] = None

        # set to None if unique_media_played50_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played50_percent is None and "unique_media_played50_percent" in self.__fields_set__:
            _dict['uniqueMediaPlayed50Percent'] = None

        # set to None if unique_media_played75_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played75_percent is None and "unique_media_played75_percent" in self.__fields_set__:
            _dict['uniqueMediaPlayed75Percent'] = None

        # set to None if unique_media_played100_percent (nullable) is None
        # and __fields_set__ contains the field
        if self.unique_media_played100_percent is None and "unique_media_played100_percent" in self.__fields_set__:
            _dict['uniqueMediaPlayed100Percent'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetStatisticsPerUnitResponseMessage:
        """Create an instance of GetStatisticsPerUnitResponseMessage from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetStatisticsPerUnitResponseMessage.parse_obj(obj)

        _obj = GetStatisticsPerUnitResponseMessage.parse_obj({
            "seq": obj.get("seq"),
            "impression": obj.get("impression"),
            "media_played": obj.get("mediaPlayed"),
            "media_played25_percent": obj.get("mediaPlayed25Percent"),
            "media_played50_percent": obj.get("mediaPlayed50Percent"),
            "media_played75_percent": obj.get("mediaPlayed75Percent"),
            "media_played100_percent": obj.get("mediaPlayed100Percent"),
            "unique_impression": obj.get("uniqueImpression"),
            "unique_media_played": obj.get("uniqueMediaPlayed"),
            "unique_media_played25_percent": obj.get("uniqueMediaPlayed25Percent"),
            "unique_media_played50_percent": obj.get("uniqueMediaPlayed50Percent"),
            "unique_media_played75_percent": obj.get("uniqueMediaPlayed75Percent"),
            "unique_media_played100_percent": obj.get("uniqueMediaPlayed100Percent")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    VideoSendMessage
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

        self.video_message = VideoSendMessage(
            original_content_url='https://example.com/original.mp4',
            preview_image_url='https://example.com/preview.jpg',
            tracking_id='TrackId'
        )

        self.message = [{
            "type": "video",
            "originalContentUrl": "https://example.com/original.mp4",
            "previewImageUrl": "https://example.com/preview.jpg",
            "trackingId": "TrackId"
        }]

    @responses.activate
    def test_push_video_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={}, status=200
        )

        self.tested.push_message('to', self.video_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": "to",
                'notificationDisabled': False,
                "messages": self.message
            }
        )

    @responses.activate
    def test_reply_video_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply',
            json={}, status=200
        )

        self.tested.reply_message('replyToken', self.video_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/reply')
        self.assertEqual(
            json.loads(request.body),
            {
                "replyToken": "replyToken",
                "messages": self.message,
                'notificationDisabled': False,
            }
        )

    @responses.activate
    def test_multicast_video_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast',
            json={}, status=200
        )

        self.tested.multicast(['to1', 'to2'], self.video_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/multicast')
        self.assertEqual(
            json.loads(request.body),
            {
                "to": ['to1', 'to2'],
                "messages": self.message,
                "notificationDisabled": False,
            }
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import unittest
from pytest_httpserver.httpserver import HTTPServer

from linebot.v3.messaging import (
    Configuration,
    ApiClient,
    MessagingApi,
)


class TestGetFollowersWithQueryParam(unittest.TestCase):
    def test_get_followers_with_query(self):
        expected_response = {
            "userIds": [
                "U4af4980629",
                "U0c229f96c4",
                "U95afb1d4df"
            ],
            "next": "yANU9IA..."
        }

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri="/v2/bot/followers/ids",
                method="GET",
            ).respond_with_json(
                expected_response,
                status=200
            )

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)
                response = line_bot_api.get_followers()

            # Verify response
            self.assertEqual(response.user_ids, expected_response["userIds"])
            self.assertEqual(response.next, expected_response["next"])

            # Verify request details
            self.assertEqual(len(httpserver.log), 1)
            req, res = httpserver.log[0]
            self.assertEqual(req.method, "GET")
            self.assertEqual(req.path, "/v2/bot/followers/ids")
            self.assertEqual(req.query_string, b"")

    def test_get_followers_with_start(self):
        expected_response = {
            "userIds": [
                "U4af4980629",
                "U0c229f96c4",
                "U95afb1d4df"
            ],
            "next": "yANU9IA..."
        }

        with HTTPServer() as httpserver:
            httpserver.expect_request(
                uri="/v2/bot/followers/ids",
                method="GET",
            ).respond_with_json(
                expected_response,
                status=200
            )

            configuration = Configuration(
                access_token="dummy-channel-access-token",
                host=httpserver.url_for("/")
            )

            with ApiClient(configuration) as api_client:
                line_bot_api = MessagingApi(api_client)
                response = line_bot_api.get_followers(start="dummy-start", limit=123)

            # Verify response
            self.assertEqual(response.user_ids, expected_response["userIds"])
            self.assertEqual(response.next, expected_response["next"])

            # Verify request details
            self.assertEqual(len(httpserver.log), 1)
            req, res = httpserver.log[0]
            self.assertEqual(req.method, "GET")
            self.assertEqual(req.path, "/v2/bot/followers/ids")
            self.assertEqual(req.query_string, b"start=dummy-start&limit=123")


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa
"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.webhooks.models.account_link_event import AccountLinkEvent
from linebot.v3.webhooks.models.activated_event import ActivatedEvent
from linebot.v3.webhooks.models.all_mentionee import AllMentionee
from linebot.v3.webhooks.models.attached_module_content import AttachedModuleContent
from linebot.v3.webhooks.models.audio_message_content import AudioMessageContent
from linebot.v3.webhooks.models.beacon_content import BeaconContent
from linebot.v3.webhooks.models.beacon_event import BeaconEvent
from linebot.v3.webhooks.models.bot_resumed_event import BotResumedEvent
from linebot.v3.webhooks.models.bot_suspended_event import BotSuspendedEvent
from linebot.v3.webhooks.models.callback_request import CallbackRequest
from linebot.v3.webhooks.models.chat_control import ChatControl
from linebot.v3.webhooks.models.content_provider import ContentProvider
from linebot.v3.webhooks.models.deactivated_event import DeactivatedEvent
from linebot.v3.webhooks.models.delivery_context import DeliveryContext
from linebot.v3.webhooks.models.detached_module_content import DetachedModuleContent
from linebot.v3.webhooks.models.emoji import Emoji
from linebot.v3.webhooks.models.event import Event
from linebot.v3.webhooks.models.event_mode import EventMode
from linebot.v3.webhooks.models.file_message_content import FileMessageContent
from linebot.v3.webhooks.models.follow_detail import FollowDetail
from linebot.v3.webhooks.models.follow_event import FollowEvent
from linebot.v3.webhooks.models.group_source import GroupSource
from linebot.v3.webhooks.models.image_message_content import ImageMessageContent
from linebot.v3.webhooks.models.image_set import ImageSet
from linebot.v3.webhooks.models.join_event import JoinEvent
from linebot.v3.webhooks.models.joined_members import JoinedMembers
from linebot.v3.webhooks.models.joined_membership_content import JoinedMembershipContent
from linebot.v3.webhooks.models.leave_event import LeaveEvent
from linebot.v3.webhooks.models.left_members import LeftMembers
from linebot.v3.webhooks.models.left_membership_content import LeftMembershipContent
from linebot.v3.webhooks.models.link_content import LinkContent
from linebot.v3.webhooks.models.location_message_content import LocationMessageContent
from linebot.v3.webhooks.models.member_joined_event import MemberJoinedEvent
from linebot.v3.webhooks.models.member_left_event import MemberLeftEvent
from linebot.v3.webhooks.models.membership_content import MembershipContent
from linebot.v3.webhooks.models.membership_event import MembershipEvent
from linebot.v3.webhooks.models.mention import Mention
from linebot.v3.webhooks.models.mentionee import Mentionee
from linebot.v3.webhooks.models.message_content import MessageContent
from linebot.v3.webhooks.models.message_event import MessageEvent
from linebot.v3.webhooks.models.module_content import ModuleContent
from linebot.v3.webhooks.models.module_event import ModuleEvent
from linebot.v3.webhooks.models.pnp_delivery import PnpDelivery
from linebot.v3.webhooks.models.pnp_delivery_completion_event import PnpDeliveryCompletionEvent
from linebot.v3.webhooks.models.postback_content import PostbackContent
from linebot.v3.webhooks.models.postback_event import PostbackEvent
from linebot.v3.webhooks.models.renewed_membership_content import RenewedMembershipContent
from linebot.v3.webhooks.models.room_source import RoomSource
from linebot.v3.webhooks.models.source import Source
from linebot.v3.webhooks.models.sticker_message_content import StickerMessageContent
from linebot.v3.webhooks.models.text_message_content import TextMessageContent
from linebot.v3.webhooks.models.unfollow_event import UnfollowEvent
from linebot.v3.webhooks.models.unsend_detail import UnsendDetail
from linebot.v3.webhooks.models.unsend_event import UnsendEvent
from linebot.v3.webhooks.models.user_mentionee import UserMentionee
from linebot.v3.webhooks.models.user_source import UserSource
from linebot.v3.webhooks.models.video_message_content import VideoMessageContent
from linebot.v3.webhooks.models.video_play_complete import VideoPlayComplete
from linebot.v3.webhooks.models.video_play_complete_event import VideoPlayCompleteEvent



# coding: utf-8

# flake8: noqa
"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.module.models.acquire_chat_control_request import AcquireChatControlRequest
from linebot.v3.module.models.detach_module_request import DetachModuleRequest
from linebot.v3.module.models.get_modules_response import GetModulesResponse
from linebot.v3.module.models.module_bot import ModuleBot



# coding: utf-8

# flake8: noqa
"""
    Mission Stickers API

    This document describes LINE Mission Stickers API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.shop.models.error_response import ErrorResponse
from linebot.v3.shop.models.mission_sticker_request import MissionStickerRequest



# coding: utf-8

# flake8: noqa
"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.audience.models.adaccount import Adaccount
from linebot.v3.audience.models.add_audience_to_audience_group_request import AddAudienceToAudienceGroupRequest
from linebot.v3.audience.models.audience import Audience
from linebot.v3.audience.models.audience_group import AudienceGroup
from linebot.v3.audience.models.audience_group_create_route import AudienceGroupCreateRoute
from linebot.v3.audience.models.audience_group_failed_type import AudienceGroupFailedType
from linebot.v3.audience.models.audience_group_job import AudienceGroupJob
from linebot.v3.audience.models.audience_group_job_failed_type import AudienceGroupJobFailedType
from linebot.v3.audience.models.audience_group_job_status import AudienceGroupJobStatus
from linebot.v3.audience.models.audience_group_job_type import AudienceGroupJobType
from linebot.v3.audience.models.audience_group_permission import AudienceGroupPermission
from linebot.v3.audience.models.audience_group_status import AudienceGroupStatus
from linebot.v3.audience.models.audience_group_type import AudienceGroupType
from linebot.v3.audience.models.create_audience_group_request import CreateAudienceGroupRequest
from linebot.v3.audience.models.create_audience_group_response import CreateAudienceGroupResponse
from linebot.v3.audience.models.create_click_based_audience_group_request import CreateClickBasedAudienceGroupRequest
from linebot.v3.audience.models.create_click_based_audience_group_response import CreateClickBasedAudienceGroupResponse
from linebot.v3.audience.models.create_imp_based_audience_group_request import CreateImpBasedAudienceGroupRequest
from linebot.v3.audience.models.create_imp_based_audience_group_response import CreateImpBasedAudienceGroupResponse
from linebot.v3.audience.models.detailed_owner import DetailedOwner
from linebot.v3.audience.models.error_detail import ErrorDetail
from linebot.v3.audience.models.error_response import ErrorResponse
from linebot.v3.audience.models.get_audience_data_response import GetAudienceDataResponse
from linebot.v3.audience.models.get_audience_groups_response import GetAudienceGroupsResponse
from linebot.v3.audience.models.get_shared_audience_data_response import GetSharedAudienceDataResponse
from linebot.v3.audience.models.get_shared_audience_groups_response import GetSharedAudienceGroupsResponse
from linebot.v3.audience.models.update_audience_group_description_request import UpdateAudienceGroupDescriptionRequest



# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List
from pydantic.v1 import Field, StrictStr, conlist
from linebot.v3.webhooks.models.module_content import ModuleContent

class AttachedModuleContent(ModuleContent):
    """
    AttachedModuleContent
    """
    bot_id: StrictStr = Field(..., alias="botId", description="User ID of the bot on the attached LINE Official Account")
    scopes: conlist(StrictStr) = Field(..., description="An array of strings indicating the scope permitted by the admin of the LINE Official Account.")
    type: str = "attached"

    __properties = ["type", "botId", "scopes"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AttachedModuleContent:
        """Create an instance of AttachedModuleContent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AttachedModuleContent:
        """Create an instance of AttachedModuleContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AttachedModuleContent.parse_obj(obj)

        _obj = AttachedModuleContent.parse_obj({
            "type": obj.get("type"),
            "bot_id": obj.get("botId"),
            "scopes": obj.get("scopes")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import Field, StrictStr
from linebot.v3.messaging.models.imagemap_action import ImagemapAction
from linebot.v3.messaging.models.imagemap_area import ImagemapArea

class MessageImagemapAction(ImagemapAction):
    """
    MessageImagemapAction
    """
    text: StrictStr = Field(...)
    label: Optional[StrictStr] = None
    type: str = "message"

    __properties = ["type", "area", "text", "label"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MessageImagemapAction:
        """Create an instance of MessageImagemapAction from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of area
        if self.area:
            _dict['area'] = self.area.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MessageImagemapAction:
        """Create an instance of MessageImagemapAction from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MessageImagemapAction.parse_obj(obj)

        _obj = MessageImagemapAction.parse_obj({
            "type": obj.get("type"),
            "area": ImagemapArea.from_dict(obj.get("area")) if obj.get("area") is not None else None,
            "text": obj.get("text"),
            "label": obj.get("label")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.v3.utils import to_camel_case, to_snake_case, safe_compare_digest


class TestUtils(unittest.TestCase):
    def test_to_snake_case(self):
        self.assertEqual(to_snake_case('hogeBarFuga'), 'hoge_bar_fuga')
        self.assertEqual(to_snake_case('uniqueMediaPlayed100Percent'), 'unique_media_played_100_percent')
        self.assertEqual(to_snake_case('festival20Days'), 'festival_20_days')
        self.assertEqual(to_snake_case('festival20days'), 'festival_20_days')

    def test_to_camel_case(self):
        self.assertEqual(to_camel_case('hoge_bar'), 'hogeBar')
        self.assertEqual(to_camel_case('unique_media_played_100_percent'), 'uniqueMediaPlayed100Percent')

    def test_safe_compare_digest_true(self):
        self.assertTrue(safe_compare_digest('/gg9a+LvFevTH1sd7', '/gg9a+LvFevTH1sd7'))

    def test_safe_compare_digest_false_same_size(self):
        self.assertFalse(safe_compare_digest('/gg9a+LvFevTH1sd7', '/gg9a+LvFevTH1sd8'))

    def test_safe_compare_digest_false_different_size(self):
        self.assertFalse(safe_compare_digest('/gg9a+LvFevTH1sd7', '/gg9a+LvFevTH1sd78'))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.flex_message module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .background import Background, LinearGradientBackground
from .actions import get_action
from .base import Base
from .send_messages import SendMessage


class FlexSendMessage(SendMessage):
    """FlexSendMessage.

    https://developers.line.biz/en/reference/messaging-api/#flex-message

    Flex Messages are messages with a customizable layout.
    You can customize the layout freely by combining multiple elements.
    """

    def __init__(self, alt_text=None, contents=None, **kwargs):
        """__init__ method.

        :param str alt_text: Alternative text
        :param contents: Flex Message container object
        :type contents: :py:class:`linebot.models.flex_message.FlexContainer`
        :param kwargs:
        """
        super(FlexSendMessage, self).__init__(**kwargs)

        self.type = 'flex'
        self.alt_text = alt_text
        self.contents = self.get_or_new_from_json_dict_with_types(
            contents, {
                'bubble': BubbleContainer,
                'carousel': CarouselContainer
            }
        )


class FlexContainer(with_metaclass(ABCMeta, Base)):
    """FlexContainer.

    https://developers.line.biz/en/reference/messaging-api/#container

    A container is the top-level structure of a Flex Message.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(FlexContainer, self).__init__(**kwargs)

        self.type = None


class BubbleContainer(FlexContainer):
    """BubbleContainer.

    https://developers.line.biz/en/reference/messaging-api/#bubble

    This is a container that contains one message bubble.
    It can contain four blocks: header, hero, body, and footer.
    """

    def __init__(self, size=None, direction=None, header=None, hero=None,
                 body=None, footer=None, styles=None, action=None, **kwargs):
        """__init__ method.

        :param str size: The size of the bubble. `nano`, `micro`, `kilo`, `mega`, or `giga`.
        :param str direction: Text directionality and the order of components
            in horizontal boxes in the container
        :param header: Header block
        :type header: :py:class:`linebot.models.flex_message.BoxComponent`
        :param hero: Hero block
        :type hero: :py:class:`linebot.models.flex_message.ImageComponent`
            | :py:class:`linebot.models.flex_message.BoxComponent`
            | :py:class:`linebot.models.flex_message.VideoComponent`
        :param body: Body block
        :type body: :py:class:`linebot.models.flex_message.BoxComponent`
        :param footer: Footer block
        :type footer: :py:class:`linebot.models.flex_message.BoxComponent`
        :param styles: Style of each block
        :type styles: :py:class:`linebot.models.flex_message.BubbleStyle`
        :param action: Action performed when this button is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param kwargs:
        """
        super(BubbleContainer, self).__init__(**kwargs)

        self.type = 'bubble'
        self.size = size
        self.direction = direction
        self.header = self.get_or_new_from_json_dict(header, BoxComponent)
        self.hero = self.get_or_new_from_json_dict_with_types(
            hero, {
                'image': ImageComponent,
                'box': BoxComponent,
                'video': VideoComponent
            }
        )
        self.body = self.get_or_new_from_json_dict(body, BoxComponent)
        self.footer = self.get_or_new_from_json_dict(footer, BoxComponent)
        self.styles = self.get_or_new_from_json_dict(styles, BubbleStyle)
        self.action = get_action(action)


class BubbleStyle(with_metaclass(ABCMeta, Base)):
    """BubbleStyle.

    https://developers.line.biz/en/reference/messaging-api/#bubble-style
    """

    def __init__(self, header=None, hero=None, body=None, footer=None, **kwargs):
        """__init__ method.

        :param header: Style of the header block
        :type header: :py:class:`linebot.models.flex_message.BlockStyle`
        :param hero: Style of the hero block
        :type hero: :py:class:`linebot.models.flex_message.BlockStyle`
        :param body: Style of the body block
        :type body: :py:class:`linebot.models.flex_message.BlockStyle`
        :param footer: Style of the footer block
        :type footer: :py:class:`linebot.models.flex_message.BlockStyle`
        :param kwargs:
        """
        super(BubbleStyle, self).__init__(**kwargs)

        self.header = self.get_or_new_from_json_dict(header, BlockStyle)
        self.hero = self.get_or_new_from_json_dict(hero, BlockStyle)
        self.body = self.get_or_new_from_json_dict(body, BlockStyle)
        self.footer = self.get_or_new_from_json_dict(footer, BlockStyle)


class BlockStyle(with_metaclass(ABCMeta, Base)):
    """BlockStyle.

    https://developers.line.biz/en/reference/messaging-api/#block-style
    """

    def __init__(self, background_color=None, separator=None, separator_color=None, **kwargs):
        """__init__ method.

        :param str background_color: Background color of the block. Use a hexadecimal color code
        :param bool separator: True to place a separator above the block
            True will be ignored for the first block in a container
            because you cannot place a separator above the first block.
            The default value is False
        :param str separator_color: Color of the separator. Use a hexadecimal color code
        :param kwargs:
        """
        super(BlockStyle, self).__init__(**kwargs)
        self.background_color = background_color
        self.separator = separator
        self.separator_color = separator_color


class CarouselContainer(FlexContainer):
    """CarouselContainer.

    https://developers.line.biz/en/reference/messaging-api/#f-carousel

    This is a container that contains multiple bubble containers, or message bubbles.
    The bubbles will be shown in order by scrolling horizontally.
    """

    def __init__(self, contents=None, **kwargs):
        """__init__ method.

        :param contents: Array of bubble containers
        :type contents: list[T <= :py:class:`linebot.models.flex_message.BubbleContainer`]
        :param kwargs:
        """
        super(CarouselContainer, self).__init__(**kwargs)

        self.type = 'carousel'

        new_contents = []
        if contents:
            for it in contents:
                new_contents.append(self.get_or_new_from_json_dict(
                    it, BubbleContainer
                ))
        self.contents = new_contents


class FlexComponent(with_metaclass(ABCMeta, Base)):
    """FlexComponent.

    https://developers.line.biz/en/reference/messaging-api/#component

    Components are objects that compose a Flex Message container.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(FlexComponent, self).__init__(**kwargs)

        self.type = None


class BoxComponent(FlexComponent):
    """BoxComponent.

    https://developers.line.biz/en/reference/messaging-api/#box

    This is a component that defines the layout of child components.
    You can also include a box in a box.
    """

    def __init__(self,
                 layout=None,
                 contents=None,
                 background_color=None,
                 border_color=None,
                 border_width=None,
                 corner_radius=None,
                 justify_content=None,
                 align_items=None,
                 background=None,
                 width=None,
                 max_width=None,
                 height=None,
                 max_height=None,
                 flex=None,
                 spacing=None,
                 margin=None,
                 padding_all=None,
                 padding_top=None,
                 padding_bottom=None,
                 padding_start=None,
                 padding_end=None,
                 position=None,
                 offset_top=None,
                 offset_bottom=None,
                 offset_start=None,
                 offset_end=None,
                 action=None,
                 **kwargs):
        """__init__ method.

        :param str layout: The placement style of components in this box
        :param contents: Components in this box
        :type contents: list[T <= :py:class:`linebot.models.flex_message.FlexComponent`]
        :param str background_color: Background color of the block
        :param str border_color: Color of box border
        :param str border_width: Width of box border
        :param str corner_radius: Radius at the time of rounding the corners of the border
        :param str justify_content: How child elements are aligned along the main axis of
            the parent element
        :param str align_items: How child elements are aligned along the cross axis of
            the parent element
        :param background: Background object
        :type background: T <= :py:class:`linebot.models.background.Background`
        :param str width: Width of the box
        :param str max_width: Maximum width of the box
        :param str height: Height of the box
        :param str max_height: Maximum height of the box
        :param float flex: The ratio of the width or height of this box within the parent box
            and the previous component in the parent box
        :param str spacing: Minimum space between components in this box
        :param str margin: Minimum space between this box
        :param str padding_all: Free space between the borders of this box and the child element
        :param str padding_top: Free space between the border at the upper end of this box
            and the upper end of the child element
        :param str padding_bottom: Free space between the border at the lower end of this box
            and the lower end of the child element
        :param str padding_start: Free space between the border at the left end of this box
            and the left end of the child element
        :param str padding_end: Free space between the border at the right end of this box
            and the right end of the child element
        :param str position: Reference position for placing this box
        :param str offset_top: The top offset
        :param str offset_bottom: The bottom offset
        :param str offset_start: The left offset
        :param str offset_end: The right offset
        :param action: Action performed when this button is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param kwargs:
        """
        super(BoxComponent, self).__init__(**kwargs)
        self.type = 'box'
        self.layout = layout
        self.background_color = background_color
        self.border_color = border_color
        self.border_width = border_width
        self.corner_radius = corner_radius
        self.justify_content = justify_content
        self.align_items = align_items
        self.width = width
        self.max_width = max_width
        self.height = height
        self.max_height = max_height
        self.flex = flex
        self.spacing = spacing
        self.margin = margin
        self.padding_all = padding_all
        self.padding_top = padding_top
        self.padding_bottom = padding_bottom
        self.padding_start = padding_start
        self.padding_end = padding_end
        self.position = position
        self.offset_top = offset_top
        self.offset_bottom = offset_bottom
        self.offset_start = offset_start
        self.offset_end = offset_end
        self.action = get_action(action)
        self.background = Background.get_or_new_from_json_dict_with_types(
            background, {'linearGradient': LinearGradientBackground}
        )

        new_contents = []
        if contents:
            for it in contents:
                new_contents.append(self.get_or_new_from_json_dict_with_types(
                    it, {
                        'box': BoxComponent,
                        'button': ButtonComponent,
                        'filler': FillerComponent,
                        'icon': IconComponent,
                        'image': ImageComponent,
                        'span': SpanComponent,
                        'separator': SeparatorComponent,
                        'text': TextComponent,
                        'video': VideoComponent
                    }
                ))
        self.contents = new_contents


class ButtonComponent(FlexComponent):
    """ButtonComponent.

    https://developers.line.biz/en/reference/messaging-api/#button

    This component draws a button.
    When the user taps a button, a specified action is performed.
    """

    def __init__(self,
                 action=None,
                 flex=None,
                 margin=None,
                 position=None,
                 offset_top=None,
                 offset_bottom=None,
                 offset_start=None,
                 offset_end=None,
                 height=None,
                 style=None,
                 color=None,
                 gravity=None,
                 adjust_mode=None,
                 **kwargs):
        """__init__ method.

        :param action: Action performed when this button is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param float flex: The ratio of the width or height of this component within the parent box
        :param str margin: Minimum space between this component
            and the previous component in the parent box
        :param str position: Reference position for placing this box
        :param str offset_top: The top offset
        :param str offset_bottom: The bottom offset
        :param str offset_start: The left offset
        :param str offset_end: The right offset
        :param str height: Height of the button
        :param str style: Style of the button
        :param str color: Character color when the style property is link.
            Background color when the style property is primary or secondary.
            Use a hexadecimal color code
        :param str gravity: Vertical alignment style
        :param str adjust_mode: The method by which to adjust the text font size
        :param kwargs:
        """
        super(ButtonComponent, self).__init__(**kwargs)
        self.type = 'button'
        self.action = get_action(action)
        self.flex = flex
        self.margin = margin
        self.position = position
        self.offset_top = offset_top
        self.offset_bottom = offset_bottom
        self.offset_start = offset_start
        self.offset_end = offset_end
        self.height = height
        self.style = style
        self.color = color
        self.gravity = gravity
        self.adjust_mode = adjust_mode


class FillerComponent(FlexComponent):
    """FillerComponent.

    https://developers.line.biz/en/reference/messaging-api/#filler

    This is an invisible component to fill extra space between components.
    """

    def __init__(self, flex=None, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(FillerComponent, self).__init__(**kwargs)
        self.type = 'filler'
        self.flex = flex


class IconComponent(FlexComponent):
    """IconComponent.

    https://developers.line.biz/en/reference/messaging-api/#icon

    This component draws an icon.
    """

    def __init__(self,
                 url=None,
                 margin=None,
                 position=None,
                 offset_top=None,
                 offset_bottom=None,
                 offset_start=None,
                 offset_end=None,
                 size=None,
                 aspect_ratio=None,
                 **kwargs):
        """__init__ method.

        :param str url: Image URL
            Protocol: HTTPS
            Image format: JPEG or PNG
        :param str margin: Minimum space between this component
            and the previous component in the parent box
        :param str position: Reference position for placing this box
        :param str offset_top: The top offset
        :param str offset_bottom: The bottom offset
        :param str offset_start: The left offset
        :param str offset_end: The right offset
        :param str size: Maximum size of the icon width
        :param str aspect_ratio: Aspect ratio of the icon
        :param kwargs:
        """
        super(IconComponent, self).__init__(**kwargs)
        self.type = 'icon'
        self.url = url
        self.margin = margin
        self.position = position
        self.offset_top = offset_top
        self.offset_bottom = offset_bottom
        self.offset_start = offset_start
        self.offset_end = offset_end
        self.size = size
        self.aspect_ratio = aspect_ratio


class ImageComponent(FlexComponent):
    """ImageComponent.

    https://developers.line.biz/en/reference/messaging-api/#f-image

    This component draws an image.
    """

    def __init__(self,
                 url=None,
                 flex=None,
                 margin=None,
                 position=None,
                 offset_top=None,
                 offset_bottom=None,
                 offset_start=None,
                 offset_end=None,
                 align=None,
                 gravity=None,
                 size=None,
                 aspect_ratio=None,
                 aspect_mode=None,
                 background_color=None,
                 action=None,
                 animated=False,
                 **kwargs):
        """__init__ method.

        :param str url: Image URL
            Protocol: HTTPS
            Image format: JPEG or PNG
        :param float flex: The ratio of the width or height of this component within the parent box
        :param str margin: Minimum space between this component
            and the previous component in the parent box
        :param str position: Reference position for placing this box
        :param str offset_top: The top offset
        :param str offset_bottom: The bottom offset
        :param str offset_start: The left offset
        :param str offset_end: The right offset
        :param str align: Horizontal alignment style
        :param str gravity: Vertical alignment style
        :param str size: Maximum size of the image width
        :param str aspect_ratio: Aspect ratio of the image
        :param str aspect_mode: Style of the image
        :param str background_color: Background color of the image. Use a hexadecimal color code.
        :param action: Action performed when this image is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param bool animated: True to play an animated image. Default is False.
        :param kwargs:
        """
        super(ImageComponent, self).__init__(**kwargs)
        self.type = 'image'
        self.url = url
        self.flex = flex
        self.margin = margin
        self.position = position
        self.offset_top = offset_top
        self.offset_bottom = offset_bottom
        self.offset_start = offset_start
        self.offset_end = offset_end
        self.align = align
        self.gravity = gravity
        self.size = size
        self.aspect_ratio = aspect_ratio
        self.aspect_mode = aspect_mode
        self.background_color = background_color
        self.action = get_action(action)
        self.animated = animated


class SeparatorComponent(FlexComponent):
    """SeparatorComponent.

    https://developers.line.biz/en/reference/messaging-api/#separator

    This component draws a separator between components in the parent box.
    """

    def __init__(self, margin=None, color=None, **kwargs):
        """__init__ method.

        :param str margin: Minimum space between this component
            and the previous component in the parent box
        :param str color: Color of the separator. Use a hexadecimal color code
        :param kwargs:
        """
        super(SeparatorComponent, self).__init__(**kwargs)
        self.type = 'separator'
        self.margin = margin
        self.color = color


class SpanComponent(FlexComponent):
    """SpanComponent.

    https://developers.line.biz/en/reference/messaging-api/#span

    This component renders multiple text strings with different designs in one row.
    """

    def __init__(self,
                 text=None,
                 color=None,
                 size=None,
                 weight=None,
                 style=None,
                 decoration=None,
                 **kwargs):
        r"""__init__ method.

        :param str text: Text
        :param str color: Font color
        :param str size: Font size
        :param str weight: Font weight
        :param str style: Style of the text
        :param str decoration: Decoration of the text
        :param kwargs:
        """
        super(SpanComponent, self).__init__(**kwargs)
        self.type = 'span'
        self.text = text
        self.size = size
        self.weight = weight
        self.color = color
        self.style = style
        self.decoration = decoration


class TextComponent(FlexComponent):
    """TextComponent.

    https://developers.line.biz/en/reference/messaging-api/#f-text

    This component draws text. You can format the text.
    """

    def __init__(self,
                 text=None,
                 contents=None,
                 flex=None,
                 margin=None,
                 position=None,
                 offset_top=None,
                 offset_bottom=None,
                 offset_start=None,
                 offset_end=None,
                 size=None,
                 align=None,
                 gravity=None,
                 wrap=None,
                 line_spacing=None,
                 max_lines=None,
                 weight=None,
                 color=None,
                 action=None,
                 style=None,
                 decoration=None,
                 **kwargs):
        r"""__init__ method.

        :param str text: Text
        :param contents: Array of spans
        :type contents: list[T <= :py:class:`linebot.models.flex_message.SpanComponent`]
        :param float flex: The ratio of the width or height of this component within the parent box
        :param str margin: Minimum space between this component
            and the previous component in the parent box
        :param str position: Reference position for placing this box
        :param str offset_top: The top offset
        :param str offset_bottom: The bottom offset
        :param str offset_start: The left offset
        :param str offset_end: The right offset
        :param str size: Font size
        :param str align: Horizontal alignment style
        :param str gravity: Vertical alignment style
        :param bool wrap: rue to wrap text. The default value is False.
            If set to True, you can use a new line character (\n) to begin on a new line.
        :param str line_spacing: Line spacing in a wrapping text
        :param int max_lines: Max number of lines
        :param str weight: Font weight
        :param str color: Font color
        :param action: Action performed when this image is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param str style: Style of the text
        :param str decoration: Decoration of the text
        :param kwargs:
        """
        super(TextComponent, self).__init__(**kwargs)
        self.type = 'text'
        self.text = text
        self.flex = flex
        self.margin = margin
        self.position = position
        self.offset_top = offset_top
        self.offset_bottom = offset_bottom
        self.offset_start = offset_start
        self.offset_end = offset_end
        self.size = size
        self.align = align
        self.gravity = gravity
        self.wrap = wrap
        self.line_spacing = line_spacing
        self.max_lines = max_lines
        self.weight = weight
        self.color = color
        self.action = get_action(action)
        self.style = style
        self.decoration = decoration

        if contents:
            self.contents = [self.get_or_new_from_json_dict(it, SpanComponent) for it in contents]
        else:
            self.contents = None


class VideoComponent(FlexComponent):
    """VideoComponent.

    https://developers.line.biz/en/reference/messaging-api/#f-video

    This component renders a video.
    """

    def __init__(self,
                 url=None,
                 preview_url=None,
                 alt_content=None,
                 aspect_ratio=None,
                 action=None,
                 **kwargs):
        r"""__init__ method.

        :param str url: URL of video file
        :param str preview_url: URL of preview image
        :param alt_content: Alternative content
        :type alt_content: :py:class:`linebot.models.flex_message.ImageComponent`
            | :py:class:`linebot.models.flex_message.BoxComponent`
        :param float aspect_ratio: Aspect ratio of the video
        :param action: Action performed when this video is tapped
        :type action: list[T <= :py:class:`linebot.models.actions.Action`]
        :param kwargs:
        """
        super(VideoComponent, self).__init__(**kwargs)

        self.type = 'video'
        self.url = url
        self.preview_url = preview_url
        self.alt_content = self.get_or_new_from_json_dict_with_types(
            alt_content, {
                'image': ImageComponent,
                'box': BoxComponent
            }
        )
        self.aspect_ratio = aspect_ratio
        self.action = get_action(action)



# coding: utf-8

# flake8: noqa
"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.messaging.models.acquisition_condition_request import AcquisitionConditionRequest
from linebot.v3.messaging.models.acquisition_condition_response import AcquisitionConditionResponse
from linebot.v3.messaging.models.action import Action
from linebot.v3.messaging.models.age_demographic import AgeDemographic
from linebot.v3.messaging.models.age_demographic_filter import AgeDemographicFilter
from linebot.v3.messaging.models.all_mention_target import AllMentionTarget
from linebot.v3.messaging.models.alt_uri import AltUri
from linebot.v3.messaging.models.app_type_demographic import AppTypeDemographic
from linebot.v3.messaging.models.app_type_demographic_filter import AppTypeDemographicFilter
from linebot.v3.messaging.models.area_demographic import AreaDemographic
from linebot.v3.messaging.models.area_demographic_filter import AreaDemographicFilter
from linebot.v3.messaging.models.audience_recipient import AudienceRecipient
from linebot.v3.messaging.models.audio_message import AudioMessage
from linebot.v3.messaging.models.bot_info_response import BotInfoResponse
from linebot.v3.messaging.models.broadcast_request import BroadcastRequest
from linebot.v3.messaging.models.buttons_template import ButtonsTemplate
from linebot.v3.messaging.models.camera_action import CameraAction
from linebot.v3.messaging.models.camera_roll_action import CameraRollAction
from linebot.v3.messaging.models.carousel_column import CarouselColumn
from linebot.v3.messaging.models.carousel_template import CarouselTemplate
from linebot.v3.messaging.models.cash_back_fixed_price_info_request import CashBackFixedPriceInfoRequest
from linebot.v3.messaging.models.cash_back_fixed_price_info_response import CashBackFixedPriceInfoResponse
from linebot.v3.messaging.models.cash_back_percentage_price_info_request import CashBackPercentagePriceInfoRequest
from linebot.v3.messaging.models.cash_back_percentage_price_info_response import CashBackPercentagePriceInfoResponse
from linebot.v3.messaging.models.cash_back_price_info_request import CashBackPriceInfoRequest
from linebot.v3.messaging.models.cash_back_price_info_response import CashBackPriceInfoResponse
from linebot.v3.messaging.models.chat_reference import ChatReference
from linebot.v3.messaging.models.clipboard_action import ClipboardAction
from linebot.v3.messaging.models.clipboard_imagemap_action import ClipboardImagemapAction
from linebot.v3.messaging.models.confirm_template import ConfirmTemplate
from linebot.v3.messaging.models.coupon_cash_back_reward_request import CouponCashBackRewardRequest
from linebot.v3.messaging.models.coupon_cash_back_reward_response import CouponCashBackRewardResponse
from linebot.v3.messaging.models.coupon_create_request import CouponCreateRequest
from linebot.v3.messaging.models.coupon_create_response import CouponCreateResponse
from linebot.v3.messaging.models.coupon_discount_reward_request import CouponDiscountRewardRequest
from linebot.v3.messaging.models.coupon_discount_reward_response import CouponDiscountRewardResponse
from linebot.v3.messaging.models.coupon_free_reward_request import CouponFreeRewardRequest
from linebot.v3.messaging.models.coupon_free_reward_response import CouponFreeRewardResponse
from linebot.v3.messaging.models.coupon_gift_reward_request import CouponGiftRewardRequest
from linebot.v3.messaging.models.coupon_gift_reward_response import CouponGiftRewardResponse
from linebot.v3.messaging.models.coupon_list_response import CouponListResponse
from linebot.v3.messaging.models.coupon_message import CouponMessage
from linebot.v3.messaging.models.coupon_others_reward_request import CouponOthersRewardRequest
from linebot.v3.messaging.models.coupon_others_reward_response import CouponOthersRewardResponse
from linebot.v3.messaging.models.coupon_response import CouponResponse
from linebot.v3.messaging.models.coupon_reward_request import CouponRewardRequest
from linebot.v3.messaging.models.coupon_reward_response import CouponRewardResponse
from linebot.v3.messaging.models.create_rich_menu_alias_request import CreateRichMenuAliasRequest
from linebot.v3.messaging.models.datetime_picker_action import DatetimePickerAction
from linebot.v3.messaging.models.demographic_filter import DemographicFilter
from linebot.v3.messaging.models.discount_explicit_price_info_request import DiscountExplicitPriceInfoRequest
from linebot.v3.messaging.models.discount_explicit_price_info_response import DiscountExplicitPriceInfoResponse
from linebot.v3.messaging.models.discount_fixed_price_info_request import DiscountFixedPriceInfoRequest
from linebot.v3.messaging.models.discount_fixed_price_info_response import DiscountFixedPriceInfoResponse
from linebot.v3.messaging.models.discount_percentage_price_info_request import DiscountPercentagePriceInfoRequest
from linebot.v3.messaging.models.discount_percentage_price_info_response import DiscountPercentagePriceInfoResponse
from linebot.v3.messaging.models.discount_price_info_request import DiscountPriceInfoRequest
from linebot.v3.messaging.models.discount_price_info_response import DiscountPriceInfoResponse
from linebot.v3.messaging.models.emoji import Emoji
from linebot.v3.messaging.models.emoji_substitution_object import EmojiSubstitutionObject
from linebot.v3.messaging.models.error_detail import ErrorDetail
from linebot.v3.messaging.models.error_response import ErrorResponse
from linebot.v3.messaging.models.filter import Filter
from linebot.v3.messaging.models.flex_block_style import FlexBlockStyle
from linebot.v3.messaging.models.flex_box import FlexBox
from linebot.v3.messaging.models.flex_box_background import FlexBoxBackground
from linebot.v3.messaging.models.flex_box_border_width import FlexBoxBorderWidth
from linebot.v3.messaging.models.flex_box_corner_radius import FlexBoxCornerRadius
from linebot.v3.messaging.models.flex_box_linear_gradient import FlexBoxLinearGradient
from linebot.v3.messaging.models.flex_box_padding import FlexBoxPadding
from linebot.v3.messaging.models.flex_box_spacing import FlexBoxSpacing
from linebot.v3.messaging.models.flex_bubble import FlexBubble
from linebot.v3.messaging.models.flex_bubble_styles import FlexBubbleStyles
from linebot.v3.messaging.models.flex_button import FlexButton
from linebot.v3.messaging.models.flex_carousel import FlexCarousel
from linebot.v3.messaging.models.flex_component import FlexComponent
from linebot.v3.messaging.models.flex_container import FlexContainer
from linebot.v3.messaging.models.flex_filler import FlexFiller
from linebot.v3.messaging.models.flex_icon import FlexIcon
from linebot.v3.messaging.models.flex_icon_size import FlexIconSize
from linebot.v3.messaging.models.flex_image import FlexImage
from linebot.v3.messaging.models.flex_image_size import FlexImageSize
from linebot.v3.messaging.models.flex_margin import FlexMargin
from linebot.v3.messaging.models.flex_message import FlexMessage
from linebot.v3.messaging.models.flex_offset import FlexOffset
from linebot.v3.messaging.models.flex_separator import FlexSeparator
from linebot.v3.messaging.models.flex_span import FlexSpan
from linebot.v3.messaging.models.flex_span_size import FlexSpanSize
from linebot.v3.messaging.models.flex_text import FlexText
from linebot.v3.messaging.models.flex_text_font_size import FlexTextFontSize
from linebot.v3.messaging.models.flex_video import FlexVideo
from linebot.v3.messaging.models.gender_demographic import GenderDemographic
from linebot.v3.messaging.models.gender_demographic_filter import GenderDemographicFilter
from linebot.v3.messaging.models.get_aggregation_unit_name_list_response import GetAggregationUnitNameListResponse
from linebot.v3.messaging.models.get_aggregation_unit_usage_response import GetAggregationUnitUsageResponse
from linebot.v3.messaging.models.get_followers_response import GetFollowersResponse
from linebot.v3.messaging.models.get_joined_membership_users_response import GetJoinedMembershipUsersResponse
from linebot.v3.messaging.models.get_membership_subscription_response import GetMembershipSubscriptionResponse
from linebot.v3.messaging.models.get_message_content_transcoding_response import GetMessageContentTranscodingResponse
from linebot.v3.messaging.models.get_webhook_endpoint_response import GetWebhookEndpointResponse
from linebot.v3.messaging.models.group_member_count_response import GroupMemberCountResponse
from linebot.v3.messaging.models.group_summary_response import GroupSummaryResponse
from linebot.v3.messaging.models.group_user_profile_response import GroupUserProfileResponse
from linebot.v3.messaging.models.image_carousel_column import ImageCarouselColumn
from linebot.v3.messaging.models.image_carousel_template import ImageCarouselTemplate
from linebot.v3.messaging.models.image_message import ImageMessage
from linebot.v3.messaging.models.imagemap_action import ImagemapAction
from linebot.v3.messaging.models.imagemap_area import ImagemapArea
from linebot.v3.messaging.models.imagemap_base_size import ImagemapBaseSize
from linebot.v3.messaging.models.imagemap_external_link import ImagemapExternalLink
from linebot.v3.messaging.models.imagemap_message import ImagemapMessage
from linebot.v3.messaging.models.imagemap_video import ImagemapVideo
from linebot.v3.messaging.models.issue_link_token_response import IssueLinkTokenResponse
from linebot.v3.messaging.models.limit import Limit
from linebot.v3.messaging.models.location_action import LocationAction
from linebot.v3.messaging.models.location_message import LocationMessage
from linebot.v3.messaging.models.lottery_acquisition_condition_request import LotteryAcquisitionConditionRequest
from linebot.v3.messaging.models.lottery_acquisition_condition_response import LotteryAcquisitionConditionResponse
from linebot.v3.messaging.models.mark_messages_as_read_request import MarkMessagesAsReadRequest
from linebot.v3.messaging.models.members_ids_response import MembersIdsResponse
from linebot.v3.messaging.models.membership import Membership
from linebot.v3.messaging.models.membership_list_response import MembershipListResponse
from linebot.v3.messaging.models.mention_substitution_object import MentionSubstitutionObject
from linebot.v3.messaging.models.mention_target import MentionTarget
from linebot.v3.messaging.models.message import Message
from linebot.v3.messaging.models.message_action import MessageAction
from linebot.v3.messaging.models.message_imagemap_action import MessageImagemapAction
from linebot.v3.messaging.models.message_quota_response import MessageQuotaResponse
from linebot.v3.messaging.models.messaging_api_pager_coupon_list_response import MessagingApiPagerCouponListResponse
from linebot.v3.messaging.models.multicast_request import MulticastRequest
from linebot.v3.messaging.models.narrowcast_progress_response import NarrowcastProgressResponse
from linebot.v3.messaging.models.narrowcast_request import NarrowcastRequest
from linebot.v3.messaging.models.normal_acquisition_condition_request import NormalAcquisitionConditionRequest
from linebot.v3.messaging.models.normal_acquisition_condition_response import NormalAcquisitionConditionResponse
from linebot.v3.messaging.models.number_of_messages_response import NumberOfMessagesResponse
from linebot.v3.messaging.models.operator_demographic_filter import OperatorDemographicFilter
from linebot.v3.messaging.models.operator_recipient import OperatorRecipient
from linebot.v3.messaging.models.pnp_messages_request import PnpMessagesRequest
from linebot.v3.messaging.models.postback_action import PostbackAction
from linebot.v3.messaging.models.push_message_request import PushMessageRequest
from linebot.v3.messaging.models.push_message_response import PushMessageResponse
from linebot.v3.messaging.models.quick_reply import QuickReply
from linebot.v3.messaging.models.quick_reply_item import QuickReplyItem
from linebot.v3.messaging.models.quota_consumption_response import QuotaConsumptionResponse
from linebot.v3.messaging.models.quota_type import QuotaType
from linebot.v3.messaging.models.recipient import Recipient
from linebot.v3.messaging.models.redelivery_recipient import RedeliveryRecipient
from linebot.v3.messaging.models.referral_acquisition_condition_response import ReferralAcquisitionConditionResponse
from linebot.v3.messaging.models.reply_message_request import ReplyMessageRequest
from linebot.v3.messaging.models.reply_message_response import ReplyMessageResponse
from linebot.v3.messaging.models.rich_menu_alias_list_response import RichMenuAliasListResponse
from linebot.v3.messaging.models.rich_menu_alias_response import RichMenuAliasResponse
from linebot.v3.messaging.models.rich_menu_area import RichMenuArea
from linebot.v3.messaging.models.rich_menu_batch_link_operation import RichMenuBatchLinkOperation
from linebot.v3.messaging.models.rich_menu_batch_operation import RichMenuBatchOperation
from linebot.v3.messaging.models.rich_menu_batch_progress_phase import RichMenuBatchProgressPhase
from linebot.v3.messaging.models.rich_menu_batch_progress_response import RichMenuBatchProgressResponse
from linebot.v3.messaging.models.rich_menu_batch_request import RichMenuBatchRequest
from linebot.v3.messaging.models.rich_menu_batch_unlink_all_operation import RichMenuBatchUnlinkAllOperation
from linebot.v3.messaging.models.rich_menu_batch_unlink_operation import RichMenuBatchUnlinkOperation
from linebot.v3.messaging.models.rich_menu_bounds import RichMenuBounds
from linebot.v3.messaging.models.rich_menu_bulk_link_request import RichMenuBulkLinkRequest
from linebot.v3.messaging.models.rich_menu_bulk_unlink_request import RichMenuBulkUnlinkRequest
from linebot.v3.messaging.models.rich_menu_id_response import RichMenuIdResponse
from linebot.v3.messaging.models.rich_menu_list_response import RichMenuListResponse
from linebot.v3.messaging.models.rich_menu_request import RichMenuRequest
from linebot.v3.messaging.models.rich_menu_response import RichMenuResponse
from linebot.v3.messaging.models.rich_menu_size import RichMenuSize
from linebot.v3.messaging.models.rich_menu_switch_action import RichMenuSwitchAction
from linebot.v3.messaging.models.room_member_count_response import RoomMemberCountResponse
from linebot.v3.messaging.models.room_user_profile_response import RoomUserProfileResponse
from linebot.v3.messaging.models.sender import Sender
from linebot.v3.messaging.models.sent_message import SentMessage
from linebot.v3.messaging.models.set_webhook_endpoint_request import SetWebhookEndpointRequest
from linebot.v3.messaging.models.show_loading_animation_request import ShowLoadingAnimationRequest
from linebot.v3.messaging.models.sticker_message import StickerMessage
from linebot.v3.messaging.models.subscribed_membership_plan import SubscribedMembershipPlan
from linebot.v3.messaging.models.subscribed_membership_user import SubscribedMembershipUser
from linebot.v3.messaging.models.subscription import Subscription
from linebot.v3.messaging.models.subscription_period_demographic import SubscriptionPeriodDemographic
from linebot.v3.messaging.models.subscription_period_demographic_filter import SubscriptionPeriodDemographicFilter
from linebot.v3.messaging.models.substitution_object import SubstitutionObject
from linebot.v3.messaging.models.template import Template
from linebot.v3.messaging.models.template_image_aspect_ratio import TemplateImageAspectRatio
from linebot.v3.messaging.models.template_image_size import TemplateImageSize
from linebot.v3.messaging.models.template_message import TemplateMessage
from linebot.v3.messaging.models.test_webhook_endpoint_request import TestWebhookEndpointRequest
from linebot.v3.messaging.models.test_webhook_endpoint_response import TestWebhookEndpointResponse
from linebot.v3.messaging.models.text_message import TextMessage
from linebot.v3.messaging.models.text_message_v2 import TextMessageV2
from linebot.v3.messaging.models.uri_action import URIAction
from linebot.v3.messaging.models.uri_imagemap_action import URIImagemapAction
from linebot.v3.messaging.models.update_rich_menu_alias_request import UpdateRichMenuAliasRequest
from linebot.v3.messaging.models.user_mention_target import UserMentionTarget
from linebot.v3.messaging.models.user_profile_response import UserProfileResponse
from linebot.v3.messaging.models.validate_message_request import ValidateMessageRequest
from linebot.v3.messaging.models.video_message import VideoMessage



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the 'License'); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an 'AS IS' BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

from linebot.models import (
    FlexSendMessage,
    BlockStyle,
    BubbleStyle,
    BubbleContainer,
    CarouselContainer,
    BoxComponent,
    TextComponent,
    SeparatorComponent,
    ImageComponent,
    ButtonComponent,
    FillerComponent,
    IconComponent,
    SpanComponent,
    VideoComponent,
    URIAction,
    LinearGradientBackground,
)
from tests.models.serialize_test_case import SerializeTestCase


class TestFlexMessage(SerializeTestCase):
    def test_flex_message(self):
        arg = {
            'alt_text': 'this is a flex message',
            'contents':
                BubbleContainer(
                    body=BoxComponent(
                        layout='vertical',
                        contents=[
                            TextComponent(text='hello', wrap=True, max_lines=1),
                            TextComponent(text='world')
                        ]
                    )
                )
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.FLEX),
            FlexSendMessage(**arg).as_json_dict()
        )

    def test_bubble_container(self):
        arg = {
            'header':
                BoxComponent(layout='vertical',
                             contents=[TextComponent(text='Header text')]),
            'body':
                BoxComponent(layout='vertical',
                             contents=[TextComponent(text='Body text')]),
            'footer':
                BoxComponent(layout='vertical',
                             contents=[TextComponent(text='Footer text')]),
            'styles':
                BubbleStyle(
                    header=BlockStyle(background_color='#00ffff'),
                    hero=BlockStyle(background_color='#00ffff',
                                    separator=True),
                    footer=BlockStyle(background_color='#00ffff',
                                      separator=True,
                                      separator_color='#00ffff')
                )
        }
        heros = [
            ImageComponent(uri='https://example.com/flex/images/image.jpg'),
            BoxComponent(layout='vertical',
                         contents=[TextComponent(text='Body text')]),
        ]
        for hero in heros:
            arg['hero'] = hero
            self.assertEqual(
                self.serialize_as_dict(arg, type=self.BUBBLE),
                BubbleContainer(**arg).as_json_dict()
            )

    def test_bubble_style(self):
        arg = {
            'header':
                BlockStyle(background_color='#00ffff'),
            'hero':
                BlockStyle(background_color='#00ffff',
                           separator=True),
            'footer':
                BlockStyle(background_color='#00ffff',
                           separator=True,
                           separator_color='#00ffff')
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            BubbleStyle(**arg).as_json_dict()
        )

    def test_block_style(self):
        arg = {
            'background_color': '#00ffff',
            'separator': True,
            'separator_color': '#000000'
        }
        self.assertEqual(
            self.serialize_as_dict(arg),
            BlockStyle(**arg).as_json_dict()
        )

    def test_carousel_container(self):
        arg = {
            'contents': [
                BubbleContainer(
                    body=BoxComponent(
                        layout='vertical',
                        contents=[TextComponent(text='Hey')]
                    )
                ),
                BubbleContainer(
                    body=BoxComponent(
                        layout='vertical',
                        contents=[TextComponent(text='Foo')]
                    )
                ),
            ]
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.CAROUSEL),
            CarouselContainer(**arg).as_json_dict()
        )

    def test_box_component(self):
        arg = {
            'layout': 'vertical',
            'contents': [
                ImageComponent(url='https://example.com/flex/images/image.jpg'),
                SeparatorComponent(),
                TextComponent(text='Text in the box'),
            ],
            'background_color': '#00000000',
            'border_width': 'light',
            'corner_radius': 'xs',
            'flex': 2
        }

        self.assertEqual(
            self.serialize_as_dict(arg, type=self.BOX),
            BoxComponent(**arg).as_json_dict()
        )

    def test_box_component_with_linear_gradient(self):
        arg = {
            'layout': 'vertical',
            'contents': [],
            'background_color': '#00000000',
            'border_width': 'light',
            'corner_radius': 'xs',
            'flex': 2,
            'background': LinearGradientBackground(
                angle='0deg',
                start_color='#ff0000',
                center_color='#0000ff',
                end_color='#00ff00',
                center_position='10%'
            )
        }

        self.assertEqual(
            self.serialize_as_dict(arg, type=self.BOX),
            BoxComponent(**arg).as_json_dict()
        )

    def test_button_component(self):
        arg = {
            'action':
                URIAction(label='Tap me',
                          uri='https://example.com'),
            'style': 'primary',
            'color': '#0000ff'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.BUTTON),
            ButtonComponent(**arg).as_json_dict()
        )

    def test_filler_component(self):
        arg = {
            'flex': 2
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.FILLER),
            FillerComponent(**arg).as_json_dict()
        )

    def test_icon_component(self):
        arg = {
            'url': 'https://example.com/icon/png/caution.png',
            'size': 'lg',
            'aspect_ratio': '1.91:1'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.ICON),
            IconComponent(**arg).as_json_dict()
        )

    def test_image_component(self):
        arg = {
            'url': 'https://example.com/flex/images/image.jpg',
            'size': 'full',
            'animated': False,
            'aspect_ratio': '1.91:1'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.IMAGE),
            ImageComponent(**arg).as_json_dict()
        )

    def test_separator_component(self):
        arg = {
            'color': '#000000',
            'margin': 'xxl'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.SEPARATOR),
            SeparatorComponent(**arg).as_json_dict()
        )

    def test_span_component(self):
        arg = {
            'type': 'span',
            'text': '蛙',
            'size': 'xxl',
            'weight': 'bold',
            'style': 'italic',
            'color': '#4f8f00',
            'decoration': 'underline'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.SPAN),
            SpanComponent(**arg).as_json_dict()
        )

    def test_video_component(self):
        arg = {
            'type': 'video',
            'url': 'https://example.com/video.mp4',
            "preview_url": "https://example.com/video_preview.jpg",
            "alt_content": {
                "type": "image",
                "size": "full",
                "aspect_ratio": "20:13",
                "aspect_mode": "cover",
                "animated": False,
                "url": "https://example.com/image.jpg"
            },
            "aspect_ratio": "20:13"
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.VIDEO),
            VideoComponent(**arg).as_json_dict()
        )

    def test_text_component(self):
        arg = {
            'text': 'Hello, World!',
            'size': 'xl',
            'weight': 'bold',
            'color': '#0000ff',
            'position': 'relative',
            'offset_top': '10px',
            'decoration': 'underline',
            'max_lines': 2
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.TEXT),
            TextComponent(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class LiffBotPrompt(str, Enum):
    """
    Specify the setting for bot link feature with one of the following values:  `normal`: Display the option to add the LINE Official Account as a friend in the channel consent screen. `aggressive`: Display a screen with the option to add the LINE Official Account as a friend after the channel consent screen. `none`: Don't display the option to add the LINE Official Account as a friend.   The default value is none. 
    """

    """
    allowed enum values
    """
    NORMAL = 'normal'
    AGGRESSIVE = 'aggressive'
    NONE = 'none'

    @classmethod
    def from_json(cls, json_str: str) -> LiffBotPrompt:
        """Create an instance of LiffBotPrompt from a JSON string"""
        return LiffBotPrompt(json.loads(json_str))





# coding: utf-8

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, validator

class UpdateLiffView(BaseModel):
    """
    UpdateLiffView
    https://developers.line.biz/en/reference/liff-server/#update-liff-app
    """
    type: Optional[StrictStr] = Field(None, description="Size of the LIFF app view. Specify one of these values: - compact - tall - full ")
    url: Optional[StrictStr] = Field(None, description="Endpoint URL. This is the URL of the web app that implements the LIFF app (e.g. https://example.com). Used when the LIFF app is launched using the LIFF URL. The URL scheme must be https. URL fragments (#URL-fragment) can't be specified. ")
    module_mode: Optional[StrictBool] = Field(None, alias="moduleMode", description="`true` to use the LIFF app in modular mode. When in modular mode, the action button in the header is not displayed. ")

    __properties = ["type", "url", "moduleMode"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('compact', 'tall', 'full'):
            raise ValueError("must be one of enum values ('compact', 'tall', 'full')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdateLiffView:
        """Create an instance of UpdateLiffView from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdateLiffView:
        """Create an instance of UpdateLiffView from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdateLiffView.parse_obj(obj)

        _obj = UpdateLiffView.parse_obj({
            "type": obj.get("type"),
            "url": obj.get("url"),
            "module_mode": obj.get("moduleMode")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.exceptions import (
    LineBotApiError
)
from linebot.models import (
    TextSendMessage
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        self.request_id = 'f70dd685-499a-4231-a441-f24b8d4fba21'
        self.headers = {'X-Line-Request-Id': self.request_id, 'HOGE': 'FUGA'}

    @responses.activate
    def test_error_handle(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={
                "message": "Invalid reply token"
            },
            headers=self.headers,
            status=401
        )

        try:
            self.tested.push_message('to', TextSendMessage(text='hoge'))
        except LineBotApiError as e:
            self.assertEqual(e.status_code, 401)
            self.assertEqual(e.error.message, 'Invalid reply token')
            self.assertEqual(e.request_id, self.request_id)
            self.assertEqual(e.headers['HOGE'], 'FUGA')

    @responses.activate
    def test_error_with_detail_handle(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/push',
            json={
                "message": "The request body has 2 error(s)",
                "details": [
                    {
                        "message": "May not be empty",
                        "property": "messages[0].text"
                    },
                    {
                        "message": "Must be one of the following values: [text"
                                   ", image, video, audio, location, sticker, "
                                   "richmessage, template, imagemap]",
                        "property": "messages[1].type"
                    }
                ]
            },
            headers=self.headers,
            status=400
        )

        try:
            self.tested.push_message(
                'to',
                TextSendMessage(text='hoge')
            )
        except LineBotApiError as e:
            self.assertEqual(e.status_code, 400)
            self.assertEqual(e.error.message, 'The request body has 2 error(s)')
            self.assertEqual(e.error.details[0].message, 'May not be empty')
            self.assertEqual(e.error.details[0].property, 'messages[0].text')
            self.assertEqual(
                e.error.details[1].message, "Must be one of the following "
                                            "values: [text"
                                            ", image, video, audio, "
                                            "location, sticker, "
                                            "richmessage, template, imagemap]"
            )
            self.assertEqual(e.error.details[1].property, 'messages[1].type')
            self.assertEqual(e.request_id, self.request_id)
            self.assertEqual(e.headers['HOGE'], 'FUGA')

    @responses.activate
    def test_error_handle_get_message_content(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_DATA_ENDPOINT + '/v2/bot/message/1/content',
            json={
                "message": "Invalid reply token"
            },
            headers=self.headers,
            status=404
        )

        try:
            self.tested.get_message_content(1)
        except LineBotApiError as e:
            self.assertEqual(e.status_code, 404)
            self.assertEqual(e.error.message, 'Invalid reply token')
            self.assertEqual(e.request_id, self.request_id)
            self.assertEqual(e.headers['HOGE'], 'FUGA')


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AudienceGroupJobFailedType(str, Enum):
    """
    Failed type
    """

    """
    allowed enum values
    """
    INTERNAL_ERROR = 'INTERNAL_ERROR'
    AUDIENCE_GROUP_AUDIENCE_INSUFFICIENT = 'AUDIENCE_GROUP_AUDIENCE_INSUFFICIENT'

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroupJobFailedType:
        """Create an instance of AudienceGroupJobFailedType from a JSON string"""
        return AudienceGroupJobFailedType(json.loads(json_str))





# coding: utf-8

"""
    Mission Stickers API

    This document describes LINE Mission Stickers API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr

class MissionStickerRequest(BaseModel):
    """
    Send mission stickers (v3)
    https://developers.line.biz/en/reference/partner-docs/#send-mission-stickers-v3
    """
    to: StrictStr = Field(..., description="Destination user ID")
    product_id: StrictStr = Field(..., alias="productId", description="Package ID for a set of stickers")
    product_type: StrictStr = Field(..., alias="productType", description="`STICKER`")
    send_present_message: StrictBool = Field(..., alias="sendPresentMessage", description="`false`")

    __properties = ["to", "productId", "productType", "sendPresentMessage"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MissionStickerRequest:
        """Create an instance of MissionStickerRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MissionStickerRequest:
        """Create an instance of MissionStickerRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MissionStickerRequest.parse_obj(obj)

        _obj = MissionStickerRequest.parse_obj({
            "to": obj.get("to"),
            "product_id": obj.get("productId"),
            "product_type": obj.get("productType"),
            "send_present_message": obj.get("sendPresentMessage")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.utils module."""


import logging
import re

import sys

from deprecated import deprecated

from .deprecations import (
    LineBotSdkDeprecatedIn30
)

LOGGER = logging.getLogger('linebot')

PY3 = sys.version_info[0] == 3


@deprecated(reason="Use 'from linebot.v3.utils import to_snake_case' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
def to_snake_case(text):
    """Convert to snake case.

    :param str text:
    :rtype: str
    """
    s1 = re.sub('(.)([A-Z])', r'\1_\2', text)
    s2 = re.sub('(.)([0-9]+)', r'\1_\2', s1)
    s3 = re.sub('([0-9])([a-z])', r'\1_\2', s2)
    return s3.lower()


@deprecated(reason="Use 'from linebot.v3.utils import to_camel_case' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
def to_camel_case(text):
    """Convert to camel case.

    :param str text:
    :rtype: str
    """
    split = text.split('_')
    return split[0] + "".join(x.title() for x in split[1:])


@deprecated(reason="Use 'from linebot.v3.utils import safe_compare_digest' instead. See https://github.com/line/line-bot-sdk-python/blob/master/README.rst for more details.", version='3.0.0', category=LineBotSdkDeprecatedIn30)  # noqa: E501
def safe_compare_digest(val1, val2):
    """safe_compare_digest method.

    :param val1: string or bytes for compare
    :type val1: str | bytes
    :param val2: string or bytes for compare
    :type val2: str | bytes
    """
    if len(val1) != len(val2):
        return False

    result = 0
    if PY3 and isinstance(val1, bytes) and isinstance(val2, bytes):
        for i, j in zip(val1, val2):
            result |= i ^ j
    else:
        for i, j in zip(val1, val2):
            result |= (ord(i) ^ ord(j))

    return result == 0



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr
from linebot.v3.audience.models.audience_group_job_failed_type import AudienceGroupJobFailedType
from linebot.v3.audience.models.audience_group_job_status import AudienceGroupJobStatus
from linebot.v3.audience.models.audience_group_job_type import AudienceGroupJobType

class AudienceGroupJob(BaseModel):
    """
    Audience group job
    https://developers.line.biz/en/reference/messaging-api/#get-audience-group
    """
    audience_group_job_id: Optional[StrictInt] = Field(None, alias="audienceGroupJobId", description="A job ID.")
    audience_group_id: Optional[StrictInt] = Field(None, alias="audienceGroupId", description="An audience ID.")
    description: Optional[StrictStr] = Field(None, description="The job's description.")
    type: Optional[AudienceGroupJobType] = None
    job_status: Optional[AudienceGroupJobStatus] = Field(None, alias="jobStatus")
    failed_type: Optional[AudienceGroupJobFailedType] = Field(None, alias="failedType")
    audience_count: Optional[StrictInt] = Field(None, alias="audienceCount", description="The number of accounts (recipients) that were added or removed.")
    created: Optional[StrictInt] = Field(None, description="When the job was created (in UNIX time).")

    __properties = ["audienceGroupJobId", "audienceGroupId", "description", "type", "jobStatus", "failedType", "audienceCount", "created"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroupJob:
        """Create an instance of AudienceGroupJob from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AudienceGroupJob:
        """Create an instance of AudienceGroupJob from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AudienceGroupJob.parse_obj(obj)

        _obj = AudienceGroupJob.parse_obj({
            "audience_group_job_id": obj.get("audienceGroupJobId"),
            "audience_group_id": obj.get("audienceGroupId"),
            "description": obj.get("description"),
            "type": obj.get("type"),
            "job_status": obj.get("jobStatus"),
            "failed_type": obj.get("failedType"),
            "audience_count": obj.get("audienceCount"),
            "created": obj.get("created")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictInt, StrictStr
from linebot.v3.audience.models.audience_group_create_route import AudienceGroupCreateRoute
from linebot.v3.audience.models.audience_group_failed_type import AudienceGroupFailedType
from linebot.v3.audience.models.audience_group_permission import AudienceGroupPermission
from linebot.v3.audience.models.audience_group_status import AudienceGroupStatus
from linebot.v3.audience.models.audience_group_type import AudienceGroupType

class AudienceGroup(BaseModel):
    """
    Audience group
    """
    audience_group_id: Optional[StrictInt] = Field(None, alias="audienceGroupId", description="The audience ID.")
    type: Optional[AudienceGroupType] = None
    description: Optional[StrictStr] = Field(None, description="The audience's name.")
    status: Optional[AudienceGroupStatus] = None
    failed_type: Optional[AudienceGroupFailedType] = Field(None, alias="failedType")
    audience_count: Optional[StrictInt] = Field(None, alias="audienceCount", description="The number of users included in the audience.")
    created: Optional[StrictInt] = Field(None, description="When the audience was created (in UNIX time).")
    request_id: Optional[StrictStr] = Field(None, alias="requestId", description="The request ID that was specified when the audience was created. This is only included when `audienceGroup.type` is CLICK or IMP. ")
    click_url: Optional[StrictStr] = Field(None, alias="clickUrl", description="The URL that was specified when the audience was created. This is only included when `audienceGroup.type` is CLICK and link URL is specified. ")
    is_ifa_audience: Optional[StrictBool] = Field(None, alias="isIfaAudience", description="The value indicating the type of account to be sent, as specified when creating the audience for uploading user IDs. ")
    permission: Optional[AudienceGroupPermission] = None
    create_route: Optional[AudienceGroupCreateRoute] = Field(None, alias="createRoute")

    __properties = ["audienceGroupId", "type", "description", "status", "failedType", "audienceCount", "created", "requestId", "clickUrl", "isIfaAudience", "permission", "createRoute"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroup:
        """Create an instance of AudienceGroup from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if failed_type (nullable) is None
        # and __fields_set__ contains the field
        if self.failed_type is None and "failed_type" in self.__fields_set__:
            _dict['failedType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AudienceGroup:
        """Create an instance of AudienceGroup from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AudienceGroup.parse_obj(obj)

        _obj = AudienceGroup.parse_obj({
            "audience_group_id": obj.get("audienceGroupId"),
            "type": obj.get("type"),
            "description": obj.get("description"),
            "status": obj.get("status"),
            "failed_type": obj.get("failedType"),
            "audience_count": obj.get("audienceCount"),
            "created": obj.get("created"),
            "request_id": obj.get("requestId"),
            "click_url": obj.get("clickUrl"),
            "is_ifa_audience": obj.get("isIfaAudience"),
            "permission": obj.get("permission"),
            "create_route": obj.get("createRoute")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AudienceGroupType(str, Enum):
    """
    Audience group type
    """

    """
    allowed enum values
    """
    UPLOAD = 'UPLOAD'
    CLICK = 'CLICK'
    IMP = 'IMP'
    CHAT_TAG = 'CHAT_TAG'
    FRIEND_PATH = 'FRIEND_PATH'
    RESERVATION = 'RESERVATION'
    APP_EVENT = 'APP_EVENT'
    VIDEO_VIEW = 'VIDEO_VIEW'
    WEBTRAFFIC = 'WEBTRAFFIC'
    IMAGE_CLICK = 'IMAGE_CLICK'
    RICHMENU_IMP = 'RICHMENU_IMP'
    RICHMENU_CLICK = 'RICHMENU_CLICK'

    @classmethod
    def from_json(cls, json_str: str) -> AudienceGroupType:
        """Create an instance of AudienceGroupType from a JSON string"""
        return AudienceGroupType(json.loads(json_str))





# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class FlexMargin(str, Enum):
    """
    You can specify the minimum space before a child component with the `margin` property of the child component, in pixels or with a keyword. FlexMargin just provides only keywords. 
    """

    """
    allowed enum values
    """
    NONE = 'none'
    XS = 'xs'
    SM = 'sm'
    MD = 'md'
    LG = 'lg'
    XL = 'xl'
    XXL = 'xxl'

    @classmethod
    def from_json(cls, json_str: str) -> FlexMargin:
        """Create an instance of FlexMargin from a JSON string"""
        return FlexMargin(json.loads(json_str))





# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.recipient module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class Limit(with_metaclass(ABCMeta, Base)):
    """Limit.

    https://developers.line.biz/en/reference/messaging-api/#send-narrowcast-message

    """

    def __init__(self, max=None, up_to_remaining_quota=False, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Limit, self).__init__(**kwargs)

        self.max = max
        self.up_to_remaining_quota = up_to_remaining_quota



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, conlist
from linebot.v3.audience.models.audience_group import AudienceGroup
from linebot.v3.audience.models.audience_group_job import AudienceGroupJob
from linebot.v3.audience.models.detailed_owner import DetailedOwner

class GetSharedAudienceDataResponse(BaseModel):
    """
    Get audience data
    https://developers.line.biz/en/reference/messaging-api/#get-audience-group
    """
    audience_group: Optional[AudienceGroup] = Field(None, alias="audienceGroup")
    jobs: Optional[conlist(AudienceGroupJob, max_items=50)] = Field(None, description="An array of jobs. This array is used to keep track of each attempt to add new user IDs or IFAs to an audience for uploading user IDs. Empty array is returned for any other type of audience. Max: 50 ")
    owner: Optional[DetailedOwner] = None

    __properties = ["audienceGroup", "jobs", "owner"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetSharedAudienceDataResponse:
        """Create an instance of GetSharedAudienceDataResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of audience_group
        if self.audience_group:
            _dict['audienceGroup'] = self.audience_group.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in jobs (list)
        _items = []
        if self.jobs:
            for _item in self.jobs:
                if _item:
                    _items.append(_item.to_dict())
            _dict['jobs'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of owner
        if self.owner:
            _dict['owner'] = self.owner.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetSharedAudienceDataResponse:
        """Create an instance of GetSharedAudienceDataResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetSharedAudienceDataResponse.parse_obj(obj)

        _obj = GetSharedAudienceDataResponse.parse_obj({
            "audience_group": AudienceGroup.from_dict(obj.get("audienceGroup")) if obj.get("audienceGroup") is not None else None,
            "jobs": [AudienceGroupJob.from_dict(_item) for _item in obj.get("jobs")] if obj.get("jobs") is not None else None,
            "owner": DetailedOwner.from_dict(obj.get("owner")) if obj.get("owner") is not None else None
        })
        return _obj




# coding: utf-8

"""
    Channel Access Token API

    This document describes Channel Access Token API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json



from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr

class IssueChannelAccessTokenResponse(BaseModel):
    """
    Issued channel access token
    https://developers.line.biz/en/reference/messaging-api/#issue-channel-access-token-v2-1
    """
    access_token: StrictStr = Field(..., description="Channel access token. ")
    expires_in: StrictInt = Field(..., description="Amount of time in seconds from issue to expiration of the channel access token")
    token_type: StrictStr = Field(..., description="A token type.")
    key_id: StrictStr = Field(..., description="Unique key ID for identifying the channel access token.")

    __properties = ["access_token", "expires_in", "token_type", "key_id"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IssueChannelAccessTokenResponse:
        """Create an instance of IssueChannelAccessTokenResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> IssueChannelAccessTokenResponse:
        """Create an instance of IssueChannelAccessTokenResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return IssueChannelAccessTokenResponse.parse_obj(obj)

        _obj = IssueChannelAccessTokenResponse.parse_obj({
            "access_token": obj.get("access_token"),
            "expires_in": obj.get("expires_in"),
            "token_type": obj.get("token_type") if obj.get("token_type") is not None else 'Bearer',
            "key_id": obj.get("key_id")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic.v1 import Field, StrictStr, conint

from typing import Optional

from linebot.v3.module.models.acquire_chat_control_request import AcquireChatControlRequest
from linebot.v3.module.models.detach_module_request import DetachModuleRequest
from linebot.v3.module.models.get_modules_response import GetModulesResponse

from linebot.v3.module.api_client import ApiClient
from linebot.v3.module.api_response import ApiResponse
from linebot.v3.module.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class LineModule(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://api.line.me"


    @validate_arguments
    def acquire_chat_control(self, chat_id : Annotated[StrictStr, Field(..., description="The `userId`, `roomId`, or `groupId`")], acquire_chat_control_request : Optional[AcquireChatControlRequest] = None, **kwargs) -> None:  # noqa: E501
        """acquire_chat_control  # noqa: E501

        If the Standby Channel wants to take the initiative (Chat Control), it calls the Acquire Control API. The channel that was previously an Active Channel will automatically switch to a Standby Channel.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.acquire_chat_control(chat_id, acquire_chat_control_request, async_req=True)
        >>> result = thread.get()

        :param chat_id: The `userId`, `roomId`, or `groupId` (required)
        :type chat_id: str
        :param acquire_chat_control_request:
        :type acquire_chat_control_request: AcquireChatControlRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the acquire_chat_control_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.acquire_chat_control_with_http_info(chat_id, acquire_chat_control_request, **kwargs)  # noqa: E501

    @validate_arguments
    def acquire_chat_control_with_http_info(self, chat_id : Annotated[StrictStr, Field(..., description="The `userId`, `roomId`, or `groupId`")], acquire_chat_control_request : Optional[AcquireChatControlRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """acquire_chat_control  # noqa: E501

        If the Standby Channel wants to take the initiative (Chat Control), it calls the Acquire Control API. The channel that was previously an Active Channel will automatically switch to a Standby Channel.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.acquire_chat_control_with_http_info(chat_id, acquire_chat_control_request, async_req=True)
        >>> result = thread.get()

        :param chat_id: The `userId`, `roomId`, or `groupId` (required)
        :type chat_id: str
        :param acquire_chat_control_request:
        :type acquire_chat_control_request: AcquireChatControlRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'chat_id',
            'acquire_chat_control_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method acquire_chat_control" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['chat_id']:
            _path_params['chatId'] = _params['chat_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['acquire_chat_control_request'] is not None:
            _body_params = _params['acquire_chat_control_request']

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/chat/{chatId}/control/acquire', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def detach_module(self, detach_module_request : Optional[DetachModuleRequest] = None, **kwargs) -> None:  # noqa: E501
        """detach_module  # noqa: E501

        The module channel admin calls the Detach API to detach the module channel from a LINE Official Account.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.detach_module(detach_module_request, async_req=True)
        >>> result = thread.get()

        :param detach_module_request:
        :type detach_module_request: DetachModuleRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the detach_module_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.detach_module_with_http_info(detach_module_request, **kwargs)  # noqa: E501

    @validate_arguments
    def detach_module_with_http_info(self, detach_module_request : Optional[DetachModuleRequest] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """detach_module  # noqa: E501

        The module channel admin calls the Detach API to detach the module channel from a LINE Official Account.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.detach_module_with_http_info(detach_module_request, async_req=True)
        >>> result = thread.get()

        :param detach_module_request:
        :type detach_module_request: DetachModuleRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'detach_module_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method detach_module" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['detach_module_request'] is not None:
            _body_params = _params['detach_module_request']

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/channel/detach', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_modules(self, start : Annotated[Optional[StrictStr], Field(description="Value of the continuation token found in the next property of the JSON object returned in the response. If you can't get all basic information about the bots in one request, include this parameter to get the remaining array. ")] = None, limit : Annotated[Optional[conint(strict=True, le=100)], Field(description="Specify the maximum number of bots that you get basic information from. The default value is 100. Max value: 100 ")] = None, **kwargs) -> GetModulesResponse:  # noqa: E501
        """get_modules  # noqa: E501

        Gets a list of basic information about the bots of multiple LINE Official Accounts that have attached module channels.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_modules(start, limit, async_req=True)
        >>> result = thread.get()

        :param start: Value of the continuation token found in the next property of the JSON object returned in the response. If you can't get all basic information about the bots in one request, include this parameter to get the remaining array. 
        :type start: str
        :param limit: Specify the maximum number of bots that you get basic information from. The default value is 100. Max value: 100 
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetModulesResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_modules_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_modules_with_http_info(start, limit, **kwargs)  # noqa: E501

    @validate_arguments
    def get_modules_with_http_info(self, start : Annotated[Optional[StrictStr], Field(description="Value of the continuation token found in the next property of the JSON object returned in the response. If you can't get all basic information about the bots in one request, include this parameter to get the remaining array. ")] = None, limit : Annotated[Optional[conint(strict=True, le=100)], Field(description="Specify the maximum number of bots that you get basic information from. The default value is 100. Max value: 100 ")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """get_modules  # noqa: E501

        Gets a list of basic information about the bots of multiple LINE Official Accounts that have attached module channels.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_modules_with_http_info(start, limit, async_req=True)
        >>> result = thread.get()

        :param start: Value of the continuation token found in the next property of the JSON object returned in the response. If you can't get all basic information about the bots in one request, include this parameter to get the remaining array. 
        :type start: str
        :param limit: Specify the maximum number of bots that you get basic information from. The default value is 100. Max value: 100 
        :type limit: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetModulesResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'start',
            'limit'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_modules" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('start') is not None:  # noqa: E501
            _query_params.append(('start', _params['start']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetModulesResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/list', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def release_chat_control(self, chat_id : Annotated[StrictStr, Field(..., description="The `userId`, `roomId`, or `groupId`")], **kwargs) -> None:  # noqa: E501
        """release_chat_control  # noqa: E501

        To return the initiative (Chat Control) of Active Channel to Primary Channel, call the Release Control API.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.release_chat_control(chat_id, async_req=True)
        >>> result = thread.get()

        :param chat_id: The `userId`, `roomId`, or `groupId` (required)
        :type chat_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the release_chat_control_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.release_chat_control_with_http_info(chat_id, **kwargs)  # noqa: E501

    @validate_arguments
    def release_chat_control_with_http_info(self, chat_id : Annotated[StrictStr, Field(..., description="The `userId`, `roomId`, or `groupId`")], **kwargs) -> ApiResponse:  # noqa: E501
        """release_chat_control  # noqa: E501

        To return the initiative (Chat Control) of Active Channel to Primary Channel, call the Release Control API.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.release_chat_control_with_http_info(chat_id, async_req=True)
        >>> result = thread.get()

        :param chat_id: The `userId`, `roomId`, or `groupId` (required)
        :type chat_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'chat_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method release_chat_control" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['chat_id']:
            _path_params['chatId'] = _params['chat_id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/v2/bot/chat/{chatId}/control/release', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

# flake8: noqa
"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import models into model package
from linebot.v3.insight.models.age_tile import AgeTile
from linebot.v3.insight.models.app_type_tile import AppTypeTile
from linebot.v3.insight.models.area_tile import AreaTile
from linebot.v3.insight.models.error_detail import ErrorDetail
from linebot.v3.insight.models.error_response import ErrorResponse
from linebot.v3.insight.models.gender_tile import GenderTile
from linebot.v3.insight.models.get_friends_demographics_response import GetFriendsDemographicsResponse
from linebot.v3.insight.models.get_message_event_response import GetMessageEventResponse
from linebot.v3.insight.models.get_message_event_response_click import GetMessageEventResponseClick
from linebot.v3.insight.models.get_message_event_response_message import GetMessageEventResponseMessage
from linebot.v3.insight.models.get_message_event_response_overview import GetMessageEventResponseOverview
from linebot.v3.insight.models.get_number_of_followers_response import GetNumberOfFollowersResponse
from linebot.v3.insight.models.get_number_of_message_deliveries_response import GetNumberOfMessageDeliveriesResponse
from linebot.v3.insight.models.get_statistics_per_unit_response import GetStatisticsPerUnitResponse
from linebot.v3.insight.models.get_statistics_per_unit_response_click import GetStatisticsPerUnitResponseClick
from linebot.v3.insight.models.get_statistics_per_unit_response_message import GetStatisticsPerUnitResponseMessage
from linebot.v3.insight.models.get_statistics_per_unit_response_overview import GetStatisticsPerUnitResponseOverview
from linebot.v3.insight.models.subscription_period_tile import SubscriptionPeriodTile



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, conlist
from linebot.v3.messaging.models.filter import Filter
from linebot.v3.messaging.models.limit import Limit
from linebot.v3.messaging.models.message import Message
from linebot.v3.messaging.models.recipient import Recipient

class NarrowcastRequest(BaseModel):
    """
    NarrowcastRequest
    https://developers.line.biz/en/reference/messaging-api/#send-narrowcast-message
    """
    messages: conlist(Message, max_items=5, min_items=1) = Field(..., description="List of Message objects.")
    recipient: Optional[Recipient] = None
    filter: Optional[Filter] = None
    limit: Optional[Limit] = None
    notification_disabled: Optional[StrictBool] = Field(False, alias="notificationDisabled", description="`true`: The user doesn’t receive a push notification when a message is sent. `false`: The user receives a push notification when the message is sent (unless they have disabled push notifications in LINE and/or their device). The default value is false. ")

    __properties = ["messages", "recipient", "filter", "limit", "notificationDisabled"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NarrowcastRequest:
        """Create an instance of NarrowcastRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['messages'] = _items
        # override the default output from pydantic.v1 by calling `to_dict()` of recipient
        if self.recipient:
            _dict['recipient'] = self.recipient.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of filter
        if self.filter:
            _dict['filter'] = self.filter.to_dict()
        # override the default output from pydantic.v1 by calling `to_dict()` of limit
        if self.limit:
            _dict['limit'] = self.limit.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NarrowcastRequest:
        """Create an instance of NarrowcastRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NarrowcastRequest.parse_obj(obj)

        _obj = NarrowcastRequest.parse_obj({
            "messages": [Message.from_dict(_item) for _item in obj.get("messages")] if obj.get("messages") is not None else None,
            "recipient": Recipient.from_dict(obj.get("recipient")) if obj.get("recipient") is not None else None,
            "filter": Filter.from_dict(obj.get("filter")) if obj.get("filter") is not None else None,
            "limit": Limit.from_dict(obj.get("limit")) if obj.get("limit") is not None else None,
            "notification_disabled": obj.get("notificationDisabled") if obj.get("notificationDisabled") is not None else False
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator

class GetNumberOfFollowersResponse(BaseModel):
    """
    Get number of followers
    https://developers.line.biz/en/reference/messaging-api/#get-number-of-followers
    """
    status: Optional[StrictStr] = Field(None, description="Calculation status.")
    followers: Optional[StrictInt] = Field(None, description="The number of times, as of the specified date, that a user added this LINE Official Account as a friend for the first time. The number doesn't decrease even if a user later blocks the account or when they delete their LINE account. ")
    targeted_reaches: Optional[StrictInt] = Field(None, alias="targetedReaches", description="The number of users, as of the specified date, that the LINE Official Account can reach through targeted messages based on gender, age, and/or region. This number only includes users who are active on LINE or LINE services and whose demographics have a high level of certainty. ")
    blocks: Optional[StrictInt] = Field(None, description="The number of users blocking the account as of the specified date. The number decreases when a user unblocks the account.   ")

    __properties = ["status", "followers", "targetedReaches", "blocks"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('ready', 'unready', 'out_of_service'):
            raise ValueError("must be one of enum values ('ready', 'unready', 'out_of_service')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetNumberOfFollowersResponse:
        """Create an instance of GetNumberOfFollowersResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetNumberOfFollowersResponse:
        """Create an instance of GetNumberOfFollowersResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetNumberOfFollowersResponse.parse_obj(obj)

        _obj = GetNumberOfFollowersResponse.parse_obj({
            "status": obj.get("status"),
            "followers": obj.get("followers"),
            "targeted_reaches": obj.get("targetedReaches"),
            "blocks": obj.get("blocks")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import pytest

from aiohttp import web

from linebot import (
    AsyncLineBotApi
)
from linebot.aiohttp_async_http_client import AiohttpAsyncHttpClient


@pytest.mark.asyncio
async def test_async_profile(aiohttp_client):
    expect = {
        'displayName': 'test',
        'userId': 'test',
        'language': 'en',
        'pictureUrl': 'https://obs.line-apps.com/...',
        'statusMessage': 'Hello, LINE!'
    }

    async def profile(request):
        return web.json_response(expect)

    app = web.Application()
    app.router.add_get('//v2/bot/profile/test', profile)

    aiohttp = await aiohttp_client(app, server_kwargs={"skip_url_asserts": True})
    async_client = AiohttpAsyncHttpClient(session=aiohttp)

    bot = AsyncLineBotApi('TOKENTOKEN', async_client, endpoint='/')
    profile_response = await bot.get_profile('test')
    assert profile_response.user_id == expect['userId']
    assert profile_response.display_name == expect['displayName']



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictInt, StrictStr, validator
from linebot.v3.audience.models.audience_group_type import AudienceGroupType

class CreateClickBasedAudienceGroupResponse(BaseModel):
    """
    Create audience for click-based retargeting
    https://developers.line.biz/en/reference/messaging-api/#create-click-audience-group
    """
    audience_group_id: Optional[StrictInt] = Field(None, alias="audienceGroupId", description="The audience ID.")
    type: Optional[AudienceGroupType] = None
    description: Optional[StrictStr] = Field(None, description="The audience's name.")
    created: Optional[StrictInt] = Field(None, description="When the audience was created (in UNIX time).")
    request_id: Optional[StrictStr] = Field(None, alias="requestId", description="The request ID that was specified when the audience was created.")
    click_url: Optional[StrictStr] = Field(None, alias="clickUrl", description="The URL that was specified when the audience was created.")
    create_route: Optional[StrictStr] = Field(None, alias="createRoute", description="How the audience was created. `MESSAGING_API`: An audience created with Messaging API. ")
    permission: Optional[StrictStr] = Field(None, description="Audience's update permission. Audiences linked to the same channel will be READ_WRITE.  - `READ`: Can use only. - `READ_WRITE`: Can use and update. ")
    expire_timestamp: Optional[StrictInt] = Field(None, alias="expireTimestamp", description="Time of audience expiration. Only returned for specific audiences.")
    is_ifa_audience: Optional[StrictBool] = Field(False, alias="isIfaAudience", description="The value indicating the type of account to be sent, as specified when creating the audience for uploading user IDs. One of:  true: Accounts are specified with IFAs. false (default): Accounts are specified with user IDs. ")

    __properties = ["audienceGroupId", "type", "description", "created", "requestId", "clickUrl", "createRoute", "permission", "expireTimestamp", "isIfaAudience"]

    @validator('create_route')
    def create_route_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('MESSAGING_API'):
            raise ValueError("must be one of enum values ('MESSAGING_API')")
        return value

    @validator('permission')
    def permission_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('READ', 'READ_WRITE'):
            raise ValueError("must be one of enum values ('READ', 'READ_WRITE')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateClickBasedAudienceGroupResponse:
        """Create an instance of CreateClickBasedAudienceGroupResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateClickBasedAudienceGroupResponse:
        """Create an instance of CreateClickBasedAudienceGroupResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateClickBasedAudienceGroupResponse.parse_obj(obj)

        _obj = CreateClickBasedAudienceGroupResponse.parse_obj({
            "audience_group_id": obj.get("audienceGroupId"),
            "type": obj.get("type"),
            "description": obj.get("description"),
            "created": obj.get("created"),
            "request_id": obj.get("requestId"),
            "click_url": obj.get("clickUrl"),
            "create_route": obj.get("createRoute"),
            "permission": obj.get("permission"),
            "expire_timestamp": obj.get("expireTimestamp"),
            "is_ifa_audience": obj.get("isIfaAudience") if obj.get("isIfaAudience") is not None else False
        })
        return _obj

